
/*
 * Copyright 2000-2011 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

window.name = 'tcMain';

var isIE = (navigator.appName == "Microsoft Internet Explorer") && navigator.userAgent.indexOf("Opera") == -1;
var isOpera = navigator.userAgent.indexOf("Opera") != -1;
var isIE6 = isIE && navigator.appVersion.indexOf("MSIE 6.0") != -1;

var OO = {
  extend : function(parent, extension) {
    return Object.extend(Object.extend({}, parent), extension);
  },

  bindAll : function(object) {
    var dest = new Object();
    for (var property in object) {
      if (typeof(object[property]) == "function") {
        dest[property] = object[property].bind(dest);
      } else {
        dest[property] = object[property];
      }
    }
    return dest;
  }
};

var BS = {

  Log: {
    warn:  function(msg) { if (window.console && window.console.warn)   { window.console.warn(msg) } },
    info:  function(msg) { if (window.console && window.console.info)   { window.console.info(msg) } },
    debug: function(msg) { if (window.console && window.console.debug) { window.console.debug(msg) } }
  },

  Event: {
    _observers: new Array(),
    observe: function(element, eventName, handler) {
      element = $(element);
      Event.observe(element, eventName, handler);
      this._observers.push([element, eventName, handler]);
    },

    stopObserving: function(el, eventName, handler) {
      this.cleanupDetachedElements();

      var args = $A(arguments);
      if ($(el)) {
        Event.stopObserving(el, eventName, handler);
      }
      
      var newObservers = new Array();
      for (var i=0; i<this._observers.length; i++) {
        var cached = this._observers[i];
        var equals = true;
        for (var j=0; j<args.length; j++) {
          if (args[j] != cached[j]) {
            equals = false;
            break;
          }
        }
        if (!equals) {
          newObservers.push(cached);
        }
      }
      this._observers = newObservers;
    },

    _stopObservingIf: function(func) {
      var newObservers = new Array();

      for (var i=0; i<this._observers.length; i++) {
        var cached = this._observers[i];
        var el = cached[0];
        if (func(el)) {
          //alert("stop: " + cached);
          Event.stopObserving(el, cached[1], cached[2]);
          cached[0] = null;
        } else {
          newObservers.push(cached);
        }
      }

      this._observers = newObservers;
    },

    cleanupDetachedElements: function() {
      this._stopObservingIf(function(elem) {
        return BS.Util.isDetached(elem);
      });
    },

    stopObservingInContainer: function(containerEl) {
      //var numBefore = this._observers.length;
      this.cleanupDetachedElements();

      this._stopObservingIf(function(elem) {
        return BS.Util.descendantOf(elem, containerEl);
      });

      // Cleanup jQuery handlers
      jQuery.cleanData( containerEl.getElementsByTagName("*") );
      jQuery.cleanData( [ containerEl ] );

      //var numAfter = this._observers.length;
      //BS.Log.info("removed: " + (numBefore - numAfter) + ", left: " + numAfter);
    },

    stopAll: function() {
      for (var i=0; i<this._observers.length; i++) {
        Event.stopObserving(this._observers[i][0], this._observers[i][1], this._observers[i][2]);
      }
      this._observers = new Array();
    }
  },

  loadingIcon: "<img src='" + base_uri + "/img/ajax-loader.gif' alt=''/> ",

  _canReload: true,
  _temporaryBlocksCount: 0,

  refreshBlocked: function() {
    return !BS._canReload || BS._temporaryBlocksCount > 0 ;
  },

  canReload: function() {
    return !BS.Hider.hasVisiblePopups() && !BS.refreshBlocked() &&
           document.readyState != 'loading' && document.readyState != 'uninitialized';
  },

  initReloadBlocker: function() {
    if(/Mozilla/.test(navigator.userAgent)) {
      BS.Event.observe(document.body, "mousedown", function(event) {
        var element = Event.element(event);
        if (typeof BS == 'object') {
          if (element.tagName == 'A' && element.href && element.href.indexOf("javascript://") == -1) {
            BS.blockRefreshTempopary(10);
          }
        }
      }, true);
    }

    BS.Event.observe(document.body, "mousemove", function() {
      if (typeof BS != 'object') return;
      BS.blockRefreshTempopary(3);
    }, true);

    BS.Event.observe(window, "scroll", function() {
      if (typeof BS != 'object') return;
      BS.blockRefreshTempopary(10);
    }, false);
  },

  blockRefreshTempopary: function(waitForSeconds) {
    // refresh blocked permanently ?
    if (!BS._canReload) return;

    waitForSeconds = waitForSeconds || 20;

    BS._temporaryBlocksCount++;
    setTimeout(function() {
      if (BS._temporaryBlocksCount > 0) BS._temporaryBlocksCount--;
    }, waitForSeconds * 1000);
  },

  blockRefreshPermanently: function() {
    BS._canReload = false;
  },

  unblockRefresh: function() {
    BS._canReload = true;
    BS._temporaryBlocksCount = 0;
  },

  _reloadTimeout: null,

  reload : function(force, reloadFunc) {
    if (BS._reloadTimeout) {
      clearTimeout(BS._reloadTimeout);
      BS._reloadTimeout = null;
    }

    if (BS.canReload() || force) {
      if (reloadFunc) {
        reloadFunc();
      } else {
        window.location.reload(true);
      }
    }
    else {
      BS._reloadTimeout = setTimeout(function() {
        BS.reload(force, reloadFunc);
      }, 1000);
    }
  },

  ajaxRequest: function(url, options) {
    return new Ajax.Request(url, BS._patchOptions(options, null));
  },

  ajaxUpdater: function(container, url, options) {
    return new Ajax.Updater(container, url, BS._patchOptions(options, container));
  },

  _patchOptions: function(options, container) {
    if (options == null) {
      options = {};
    }

    var oldOnComplete = null;
    if (options.onComplete != null) {
      oldOnComplete = options.onComplete.bind(options);
    }

    var elemsToClean = null;
    if (container && $(container)) {
      elemsToClean = $(container).childElements();
    }

    options.onComplete = function(transport, json) {
      try {
        var respXML = transport.responseXML;
        if (respXML) {
          var handled = BS.XMLResponse.processErrors(respXML, {
            onAccessDeniedError: function(elem) {
              BS.XMLResponse.processRedirect(elem.ownerDocument);
            }
          });
          if (handled) return;
        } else {
          var text = transport.responseText;
          if (text.match(/^\s*<!DOCTYPE html/)) {
            document.location.reload(true);
            return;
          }
        }

        if (oldOnComplete) {
          oldOnComplete(transport, json);
        }
      } finally {
        if (elemsToClean) {
          for (var i = 0; i < elemsToClean.length; i++) {
            BS.Event.stopObservingInContainer(elemsToClean[i]);
          }
          elemsToClean = null;
        } else {
          BS.Event.cleanupDetachedElements();
        }
      }
    };

    return options;
  }
};

BS.PeriodicalUpdater = Class.create(Ajax.PeriodicalUpdater, {
  initialize: function($super, container, url, options) {
    this.initialFrequency = options.frequency || 2;
    this.initialOnSuccess = options.onSuccess || Prototype.emptyFunction();
    this.initialOnFailure = options.onFailure || Prototype.emptyFunction();
    options.onSuccess = this.onSuccess.bind(this);
    options.onFailure = this.onFailure.bind(this);
    options.onException = this.onException.bind(this);
    $super(container || {update:function() {}}, url, options);
  },

  onSuccess: function(transport, json) {
    if (transport.status == 0) {
      // workaround, see http://dev.rubyonrails.org/ticket/11508
      this.onFailure(transport, json);
      return;
    }

    this.frequency = this.initialFrequency; // increase frequency
    if (BS.ServerLink) {
      BS.ServerLink.onSuccess();
    }
    if (this.initialOnSuccess) {
      this.initialOnSuccess.call(this.options, transport, json);
    }
  },

  onFailure: function(transport, json) {
    this.frequency = this.initialFrequency * 5; // reduce frequency
    if (BS.ServerLink) {
      BS.ServerLink.onFailure();
    }
    if (this.initialOnFailure) {
      this.initialOnFailure.call(this.options, transport, json);
    }
  },

  onException: function(obj, ex) {
    this.frequency = this.initialFrequency * 5; // reduce frequency
    if (BS.ServerLink) {
      BS.ServerLink.onFailure(ex);
    }
  }
});


BS.EventTracker = {
  _subscriptions : {},
  _options: {
    frequency: 6,
    evalScripts: false,
    parameters: ""
  },

  subscribeOnEvent: function(eventName, currentValue, listener) {
    this._subscribeOnEvent(eventName, currentValue, "", listener)
  },

  subscribeOnProjectEvent: function(eventName, currentValue, projectId, listener) {
    this._subscribeOnEvent(eventName, currentValue, "p:" + projectId, listener)
  },

  subscribeOnBuildTypeEvent: function(eventName, currentValue, buildTypeId, listener) {
    this._subscribeOnEvent(eventName, currentValue, "b:" + buildTypeId, listener)
  },

  subscribeOnUserEvent: function(eventName, currentValue, userId, listener) {
    this._subscribeOnEvent(eventName, currentValue, "u:" + userId, listener)
  },

  _subscribeOnEvent: function(eventName, currentValue, parameters, listener) {
    var subscr = {
      currentValue: currentValue,
      listeners: [listener],
      id: eventName + ";" + parameters
    };
    this._addSubscription(subscr);
  },

  addCustomSubscription: function(subscriptionId, listener, currentValue) {
    this._addSubscription({
      currentValue: currentValue,
      listeners: [listener],
      id: subscriptionId
    });
  },

  _addSubscription: function(subscr) {
    var curSubscr = this._subscriptions[subscr.id];
    if (curSubscr != null) {
      for (var i=0; i<curSubscr.listeners.length; i++) {
        if (curSubscr.listeners[i].toString().strip() == subscr.listeners[0].toString().strip()) return;
      };

      curSubscr.listeners.push(subscr.listeners[0]);
    } else {
      this._subscriptions[subscr.id] = subscr;
    }

    this._addParameter(subscr.id);
  },

  _addParameter: function(id) {
    if (this._options.parameters == null || this._options.parameters.length == 0) {
      this._options.parameters = "subscriptionId=";
    }

    var subs = id + ",";
    if (this._options.parameters.indexOf(subs) == -1) {
      this._options.parameters += subs;
    }
  },

  startTracking: function(trackerUrl) {
    if (!this.isStopped()) {
      this.stop();
    }

    this._trackerUrl = trackerUrl;
    this._options.onSuccess = function(transport) {
      var response = transport.responseText;
      if (response.indexOf('__tc_cleanupInProgressMarker') != -1) {
        BS.reload(true);
        return;
      }
      var listeners = new Array();
      var lines = response.split("\n");
      for (var i=0; i<lines.length; i++) {
        var line = lines[i];
        var idx = line.indexOf('=');
        if (idx == -1) continue;
        var id = line.substring(0, idx);
        var counter = line.substring(idx+1);
        var curSubscr = BS.EventTracker._subscriptions[id];
        if (curSubscr != null) {
          if (curSubscr.currentValue != counter) {
            curSubscr.currentValue = counter;
            for (var j=0; j<curSubscr.listeners.length; j++) {
              var lr = curSubscr.listeners[j];
              if (!listeners.include(lr)) {
                listeners.push(lr);
              }
            }
          }
        }
      }

      for (var k=0; k<listeners.length; k++) {
        listeners[k]();
      }
    };

    this._updater = new BS.PeriodicalUpdater(null, trackerUrl, this._options);
  },

  stop: function() {
    this._stopped = true;
    if (this._updater) {
      this._updater.stop();
    }
  },

  dispose : function() {
    this.stop();
    this._subscriptions = {};
  },

  isStopped : function() {
    return this._stopped;
  }
}


BS.StatisticsMonitor = {
  start: function(url) {
    var that = this;
    this._updater = new BS.PeriodicalUpdater(null, url, {
      frequency: 10,
      onSuccess: function(transport) {
        var doc = BS.Util.documentRoot(transport);
        var agentsNumTags = doc.getElementsByTagName("agentsNum");
        if (!agentsNumTags || agentsNumTags.length == 0) return;

        var agentsNum = that.getIntValue(agentsNumTags[0]);
        var freeAgentsNum = that.getIntValue(doc.getElementsByTagName("freeAgentsNum")[0]);
        var runningNum = that.getIntValue(doc.getElementsByTagName("runningBuildsNum")[0]);
        var buildQueueSize = that.getIntValue(doc.getElementsByTagName("buildQueueSize")[0]);

        if (agentsNum != null) {
          var agentsLoad = 0;
          if (runningNum > 0 || freeAgentsNum > 0) {
            agentsLoad = Math.round(runningNum * 100.0 / (freeAgentsNum + runningNum) );
          }
          if (agentsLoad < 0) {
            agentsLoad = 100;
          }
          var idle = freeAgentsNum;
          if (idle < 0) idle = 0;
          var title = '';
          if (agentsNum == 0) {
            title = "There are no agents";
          } else if (idle == 0) {
            title = "All agents are busy";
          } else if (idle == agentsNum) {
            title = "All agents are idle";
          } else {
            title = runningNum + (runningNum > 1 ? " builds are" : " build is") + " running, ";
            title += idle + (idle > 1 ? ' agents are' : ' agent is') + ' idle';
          }

          var content = '<div class="mug" onmouseover="BS.Tooltip.showMessage(this, {shift:{x:10,y:20}}, \'' + title + '\')" onmouseout="BS.Tooltip.hidePopup()">';
          if (agentsLoad > 0) {
            var maxheight = 16; // maximum height of the progress bar (see tabs.css)
            var top = Math.round((100 - agentsLoad) / 100 * maxheight);
            content += '<div class="mugStuff" style="top: ' + top + 'px; height: ' + (maxheight - top) + 'px;"></div>';
          }
          content += "</div>";
          var agentsTab = topNavPane.getTab("agents");
          agentsTab.setPostLinkContent(content);
          agentsTab.setCaption("Agents (" + agentsNum + ")");
        }

        if (buildQueueSize != null && topNavPane.getTab("queue")) {
          topNavPane.getTab("queue").setCaption("Build Queue (" + buildQueueSize + ")");
        }
      }
    });
  },

  getIntValue: function(element) {
    if (element != null) {
      return parseInt(element.getAttribute("value"), 10);
    }

    return null;
  }
}

BS.Util = {
  place: function(element, x, y) {
    $(element).style.left = x + "px";
    $(element).style.top = y + "px";
  },

  center: function(elementToPlace, container) {
    var pos = BS.Util.computeCenter(elementToPlace, container);
    BS.Util.place(elementToPlace, pos[0], pos[1]);
  },

  // computes x and y so that element was shown centered in the window by height
  // and centered within container by width
  computeCenter: function(elementToPlace, container) {
    if (!container) {
      container = $('mainContent');
    }
    var containerDim = Element.getDimensions(container);
    var containerPos = Position.cumulativeOffset(container);
    var windowSize = BS.Util.windowSize();

    // to obtain element dimensions we have to show it
    var oldVisibility = elementToPlace.style.visibility;
    var oldDisplay = elementToPlace.style.display;
    elementToPlace.style.visibility = 'hidden';
    elementToPlace.style.display = 'block';
    var dim = Element.getDimensions(elementToPlace);
    elementToPlace.style.visibility = oldVisibility;
    elementToPlace.style.display = oldDisplay;
    var x = 0;
    var y = Math.round(this._scrollTop() + (windowSize[1] - dim.height) / 2);
    // check if container is wider than visible area
    if (containerDim.width > windowSize[0]) {
      // position within window instead of container
      x = Math.round(this._scrollLeft() + (windowSize[0] - dim.width) / 2);
    } else {
      x = Math.round(containerPos[0] + (containerDim.width - dim.width) / 2);
    }

    return [x, y];
  },

  _scrollTop: function() {
    return window.pageYOffset ? window.pageYOffset : document.documentElement.scrollTop;
  },

  _scrollLeft: function() {
    return window.pageXOffset ? window.pageXOffset : document.documentElement.scrollLeft;
  },

  windowSize: function(win) {
    if (!win) {
      win = window;
    }
    if (win.innerWidth) {
      return [win.innerWidth, win.innerHeight];
    }

    if (win.document.documentElement && win.document.documentElement.clientWidth) {
      return [win.document.documentElement.clientWidth, win.document.documentElement.clientHeight];
    }

    if (win.document.body) {
      return [win.document.body.clientWidth, win.document.body.clientHeight];
    }

    return [0, 0];
  },

  placeNearElement: function(elementToPlace, element, shift) {
    if (!shift) {
      shift = {};
      shift.x = 0;
      shift.y = 15;
    }

    var pos = Position.positionedOffset(element);
    var x = pos[0] + shift.x;
    var y = pos[1] + shift.y;

    BS.Util.place(elementToPlace, x, y);
  },

  showNearElement : function(near_element, element_to_show, x_shift) {
    var menuDiv = $(element_to_show);
    if (typeof x_shift == 'undefined') {
      x_shift = -180;
    }
    BS.Util.placeNearElement(menuDiv, near_element, {x: x_shift, y: 21});
    BS.Hider.showDivWithTimeout(menuDiv, {hideOnMouseOut: false});
  },

  //neuro: this moved from our prototype patch
  visible: function(element) {
    if (!$(element)) return false;
    var display = Element.getStyle(element, 'display');
    return display != null && display != 'none' && display != 'null';
  },

  //neuro: this moved from our prototype patch
  show: function() {
    for (var i = 0; i < arguments.length; i++) {
      var element = $(arguments[i]);
      if (!element) continue;
      var lcTagName = element.tagName.toLowerCase();
      if ('tr' == lcTagName) {
        try {
          element.style.display = 'table-row';
        }
        catch (e) {
          // IE hack
          element.style.display = 'block';
        }
      }
      else if ('table' == lcTagName) {
        if (isIE) {
          element.style.display = 'block'; // fix for IE8
        } else {
          try {
            element.style.display = 'table';
          }
          catch (e) {
            // IE hack
            element.style.display = 'block';
          }
        }
      }
      else if ('div' == lcTagName || 'p' == lcTagName || 'dd' == lcTagName ||
               'dl' == lcTagName || 'ul' == lcTagName || 'ol' == lcTagName) {
        element.style.display = 'block';
      }
      else {
        element.style.display = '';
      }
    }
  },

  //neuro: this moved from our prototype patch
  hide: function() {
    for (var i = 0; i < arguments.length; i++) {
      var element = $(arguments[i]);
      if (element) {
        element.style.display = 'none';
      }
    }
  },

  //neuro: this is basically copy fo prototype.Form.serialize but with 1.5 contract (send disabled and all submits)
  serializeForm: function(form) {
    var elements = Form.getElements(form);
    var jQueryElementPrefix = BS.jQueryDropdown.namePrefix;
    elements = elements.filter(function(element) {
      return element.type!='password'
             && element.name.indexOf('prop:encrypted') == -1
             && !element.name.endsWith(jQueryElementPrefix);
    });

    var key, value, submitted = false, submit;

    var data = elements.inject({ }, function(result, element) {
      if (element.name) {
        key = element.name; value = $(element).getValue();
        if (value != null && (element.type != 'submit' || (!submitted &&
            submit !== false && (!submit || key == submit) && (submitted = true)))) {
          if (key in result) {
            // a key is already present; construct an array of values
            if (!Object.isArray(result[key])) result[key] = [result[key]];
            result[key].push(value);
          }
          else result[key] = value;
        }
      }
      return result;
    });

    return Object.toQueryString(data);
  },

  //neuro: using _wasDisabled to remember state
  disableFormTemp: function(form, elemsFilter) {
    var disabledElems = [];
    for (var i = 0; i < form.elements.length; i++) {
      var element = form.elements[i];
      if (!elemsFilter || elemsFilter(element)) {
        this.disableInputTemp(element);
        disabledElems.push(element);
      }
    }
    BS.VisibilityHandlers.updateVisibility(form);
    return disabledElems;
  },

  isDisabled: function(input) {
    return input.disabled == 'disabled' || input.disabled == true;
  },

  disableInputTemp: function(input) {
    input.blur();
    if (this.isDisabled(input)) {
      input._wasDisabled = true;
    } else {
      input.disabled = 'disabled';
    }
  },

  reenableInput: function(input) {
    if (typeof input._wasDisabled == 'undefined') {
      input.disabled = '';
    } else {
      input._wasDisabled = undefined;
    }
  },

  //neuro: using _wasDisabled to restore state
  reenableForm: function(form, elemsFilter) {
    for (var i = 0; i < form.elements.length; i++) {
      var element = form.elements[i];
      if (!elemsFilter || elemsFilter(element)) {
        this.reenableInput(element);
      }
    }
    BS.VisibilityHandlers.updateVisibility(form);
  },
                                                
  shiftToFitPage: function(el) {
    // Fix for TW-10259.
    var overflow = el.style.overflow;
    if (isOpera || isIE) { el.style.overflow = 'visible'; }

    var winSize = BS.Util.windowSize();
    var pos = Position.positionedOffset(el);
    var dim = Element.getDimensions(el);
    Position.prepare();
    var maxPageX = winSize[0] + Position.deltaX;
    var minElemX = pos[0];
    var maxElemX = pos[0] + dim.width;

    if (minElemX < Position.deltaX) {
      // if element is hidden by scroller
      el.style.left = (10 + Position.deltaX) + 'px';
    } else if (maxElemX > maxPageX && dim.width < maxPageX) {
      // element maximum position by X is outside visible area
      el.style.left = (maxPageX - dim.width - 20) + 'px';
    } else if (dim.width >= maxPageX) {
      // element width is more than visible area
      el.style.left = '10px';
    }

    if (isOpera || isIE) { el.style.overflow = overflow; }
  },

  showHelp: function(event, url) {
    Event.stop(event);
    var parts = url.split('#');
    if (parts.length == 2) {
      return !BS.Util.popupWindow(parts[0] + '#' + parts[1].replace(/\+/g, ''), "tcHelp");
    }
    else {
      return !BS.Util.popupWindow(url, "tcHelp");
    }
  },

  popupWindow: function(url, target, options) {
    if (!target) {
      target = "_blank";
    }

    if (!options) {
      options = {};
    }
    var width = options.width || 800;
    var height = options.height || 600;

    var w = window.open( url, target, 'toolbar=no,scrollbars=yes,resizable=yes,width=' + width + ',height=' + height);
    w.focus();
    return w;
  },

  hideSuccessMessages: function() {
    if (window._shownMessages) {
      for (var id in window._shownMessages) {
        if ($(id) && window._shownMessages[id] == 'info') {
          $(id).style.visibility = 'hidden';
        }
      }
      window._shownMessages = {};
    }
  },

  showIFrameIfNeeded: function(div) {
    var iframe = BS.Util.getOrCreateDialogIFrame(div);
    if (iframe == null) return;

    Position.absolutize(iframe);
    Position.clone(div, iframe);
    var zIndex = parseInt(div.style.zIndex, 10);
    iframe.style.zIndex = "" + (zIndex - 1);
    BS.Util.show(iframe);
  },

  hideIFrameIfNeeded: function(div) {
    var iframe = BS.Util.getOrCreateDialogIFrame(div);
    if (iframe == null) return;
    BS.Util.hide(iframe);
  },

  moveDialogIFrame: function(div) {
    var iframe = BS.Util.getOrCreateDialogIFrame(div);
    if (iframe == null) return;
    Position.absolutize(iframe);
    Position.clone(div, iframe);
  },

  getOrCreateDialogIFrame: function(div) {
    if (!isIE6) return null;
    var iframe = $('iframe_' + div.id);
    if (iframe == null) {
      iframe = document.createElement("iframe");
      iframe.src = 'javascript:false'; // this is required to disable insecure items warning in IE for https protocol
      iframe.id = 'iframe_' + div.id;
      document.body.appendChild(iframe);
    }

    return iframe;
  },

  addWordToTextArea: function(textArea, word) {
    var initialValue = textArea.value;
    if (initialValue.length == 0) {
      textArea.value = word;
    } else {
      textArea.value = initialValue + " " + word;
    }
  },

  processError: function(e) {
    if (e.message) {
      alert(e.message);
    } else {
      alert(e.toString());
    }
  },

  changeChildrenColor: function(parent, options) {
    var color = options.color;
    var bgColor = options.backgroundColor;
    var filter = options.filter;

    var childNodes = parent.childNodes;
    for (var i=0; i<childNodes.length; i++) {
      if (filter && !filter(childNodes[i])) continue;
      if (childNodes[i].style) {
        if (color != null) {
          childNodes[i].style.color = color;
        }

        if (bgColor != null) {
          childNodes[i].style.backgroundColor = bgColor;
        }
      }
    }
  },

  documentRoot: function(transport) {
    if (!transport.responseXML) return null;
    return transport.responseXML.documentElement;
  },


  /*
  * Parse response generated by ActionErrors.serialize
  * @return array of arrays: [err_id, err_message]
  * */
  xmlErrors: function(transport) {
    var root = BS.Util.documentRoot(transport);
    if (! root ) return [];
    var errs = root.getElementsByTagName("errors");
    if (!errs || errs.length == 0 || errs[0].getElementsByTagName("error").length == 0) return [];
    var errorElements = errs[0].getElementsByTagName("error");
    var result = [];
    for (var i = 0; i < errorElements.length; i ++) {
      var e = errorElements[i];
      result.push([e.getAttribute("id"), e.firstChild.nodeValue]);
    }
    return result;
  },

  trimSpaces: function(str) {
    return str.replace(/^\s+(.*)/, "$1").replace(/(.*?)\s+$/, "$1");
  },

  /*
  * Formats time in format 23h:33m:21s
  * If includeSeconds == false, seconds are not shown
  * */
  formatSeconds: function(seconds, includeSeconds) {
    if (seconds < 0) return "N/A";
    if (seconds == 0) return "&lt;1s";

    var result = "";
    var t = parseInt(seconds);

    if (t >= 3600) {
      var hours = Math.floor(t / 3600);
      t -= hours*3600;
      result += hours + "h"
    }

    if (t >= 60) {
      var mins = Math.floor(t / 60);
      t -= mins*60;
      if (result != "") {
        result += ":";
      }
      result += mins + "m";
    }

    if (t > 0 && includeSeconds) {
      seconds = t;
      if (result != "") {
        result += ":";
      }
      result += seconds + "s";
    }
    else if (result == ""){
      result = "&lt;1m";
    }
    return result;
  },

  /**
   * Turns ON all checkboxes with specified name in the specified form
   * @param form
   * @param checkboxName
   */
  selectAll: function(form, checkboxName) {
    this._setChecked(form, checkboxName, true);
  },

  /**
   * Turns OFF all checkboxes with specified name in the specified form
   * @param form
   * @param checkboxName
   */
  unselectAll: function(form, checkboxName) {
    this._setChecked(form, checkboxName, false);
  },

  /**
   * Turns array of the selected checkboxes values
   * @param form
   * @param checkboxName
   */
  getSelectedValues: function(form, checkboxName) {
    var result = [];
    var checkboxes = Form.getInputs(form, "checkbox", checkboxName);
    for (var i=0; i<checkboxes.length; i++) {
      if (checkboxes[i].checked) {
        result.push(checkboxes[i].value);
      }
    }
    return result;
  },

  _setChecked: function(form, checkboxName, checked) {
    var checkboxes = Form.getInputs(form, "checkbox", checkboxName);
    for (var i=0; i<checkboxes.length; i++) {
      if (!checkboxes[i].disabled) {
        checkboxes[i].checked = checked;
      }
    }
  },


  ie_revalidate: function(node) {
    if (!Prototype.Browser.IE) return;
    node = $(node) || $(document.body);
    if (node) {
      setTimeout(function() {
        node.addClassName("fff");
        node.removeClassName("fff");
      }, 1);
    }
  },

  descendantOf: function(child, parent) {
    while (child != null) {
      if (child === parent) return true;
      child = child.parentNode;
    }
    return false;
  },

  isDetached: function(elem) {
    if (elem.id) return $(elem.id) !== elem;

    if (elem === document.documentElement || elem === window || elem === document) return false;
    if (elem.parentNode == null) return true;

    return BS.Util.isDetached(elem.parentNode);
  },

  runWithElement: function(element_id, toRun, timeout) {
    if (!timeout) timeout = 1000;

    BS.WaitFor(function() {
      return $(element_id) && !BS.Util.isDetached($(element_id));
    }, toRun, timeout / 1000.0);
  },

  setTitle: function(title) {
    document.title = title + " -- TeamCity";
  }
};

//==========================================================================
/**
Public API:

BS.Hider.showDivWithTimeout('id');
BS.Hider.startHidingDiv('id', delay = 500);
BS.Hider.stopHidingDiv('id');
BS.Hider.hideDiv('id');


BS.Hider.hideAll('id');

*/

BS.Hider = {
  hidingDivs : {},
  allDivs : {},
  afterHideFuncs : {},

  hasVisiblePopups: function() {
    for (var v in this.allDivs) {
      return true;
    }
    return false;
  },

  _currentZindex: function() {
    if (this._shownStack.length == 0) return 10;
    var id = this._shownStack[this._shownStack.length - 1];
    return parseInt($(id).style.zIndex, 10) + 5;
  },

  addHideFuntion: function(id, hideFunction) {

    if (typeof this.afterHideFuncs[id] == 'function') {
      var old = this.afterHideFuncs[id];
      this.afterHideFuncs[id] = function() {
        hideFunction();
        old();
      }
    }
    else {
      this.afterHideFuncs[id] = hideFunction;
    }
  },

  showDivWithTimeout: function(id, options) {
    if (!options) {
      options = {};
    }

    if (options.hideOnMouseOut === undefined) {
      options.hideOnMouseOut = true;
    }

    if (options.hideOnMouseClickOutside === undefined) {
      options.hideOnMouseClickOutside = true;
    }

    if (options.draggable === undefined) {
      options.draggable = false;
    }

    if (options.dragHandle === undefined) {
      options.dragHandle = $(id);
    }

    var currentZIndex = this._currentZindex();
    if (options.zIndex === undefined || options.zIndex < currentZIndex) {
      options.zIndex = currentZIndex;
    }

    if (options.afterHideFunc != undefined) {
      this.addHideFuntion(id, options.afterHideFunc);
    }

    id = $(id).id;

    $(id).style.position = 'absolute';
    $(id).style.zIndex = "" + options.zIndex;

    BS.Util.show(id);
    BS.Util.shiftToFitPage($(id));
    BS.Util.showIFrameIfNeeded($(id));

    this.allDivs[id] = id;

    this.stopHidingDiv(id); // if was started previously

    this._shownStack.push(id);
    this._setupHandlers(id, options);

    BS.VisibilityHandlers.updateVisibility($(id));

    $(id)._hideOnMouseClickOutside = false;
    if (options.hideOnMouseClickOutside) {
      setTimeout(function(){
        $(id)._hideOnMouseClickOutside = true;
      }.bind(this), 10);
    }

    if (options.draggable) {
      $(id)._draggable = new Draggable(id, {
        starteffect: function() {},
        endeffect: function() {},
        change: function(queue) {
          BS.Util.moveDialogIFrame($(id));
        },
        handle: options.dragHandle
      });
    }
  },

  startHidingDiv: function(id, delay) {
    if (delay == undefined) {
      delay = 500;
    }

    id = $(id).id;

    this.stopHidingDiv(id, true);

    var _t = this;
    this.hidingDivs[id] = setTimeout(function() {
      if (_t.hidingDivs[id]) {
        _t.hideDiv(id);
      }
    }, delay);
  },

  hideDiv : function(id) {
    var divPos = -1;
    for (var i=0; i<this._shownStack.length; i++) {
      if (this._shownStack[i] == id) {
        divPos = i;
        break;
      }
    }
    
    if (divPos == -1) return;
    var numShown = this._shownStack.length - divPos;
    while (numShown > 0) {
      var topId = this._shownStack.pop();
      BS.Util.hideIFrameIfNeeded($(topId));
      this.stopHidingDiv(topId, true);
      delete this.allDivs[topId];
      if ($(topId) && $(topId)._draggable) {
        $(topId)._draggable.destroy();
        $(topId)._draggable = null;
      }
      BS.Event.stopObserving($(topId));
      BS.Util.hide(topId);
      if (this.afterHideFuncs[id]) {
        this.afterHideFuncs[id]();
        delete this.afterHideFuncs[id];
      }
      numShown--;
    }
  },

  stopHidingDiv : function(id, thisDivOnly) {
    var idAttr = $(id).id;
    if (this.hidingDivs[idAttr]) {
      if (thisDivOnly) {
        clearTimeout(this.hidingDivs[idAttr]);
      } else {
        for (var i=0; i<this._shownStack.length; i++) {
          clearTimeout(this._shownStack[i]);
          if (this._shownStack[i] == idAttr) {
            break;
          }
        }
      }

      delete this.hidingDivs[idAttr];
    }
  },

  hideAll: function(stopOn) {
    var divs = new Array();
    for(var divId in this.allDivs) {
      if (!$(divId)) continue;
      divs.push($(divId));
    }

    // sort divs according to their z-index
    divs.sort(function(div1, div2) {
      var zI1 = parseInt(div1.style.zIndex, 10);
      var zI2 = parseInt(div2.style.zIndex, 10);
      return zI1 - zI2;
    });

    for (var i=divs.length-1; i>=0; i--) {
      var div = divs[i];
      if (div == stopOn) break;
      if (div._hideOnMouseClickOutside == false) break;
      this.hideDiv(div.id);
      div._hideOnMouseClickOutside = true;
    }
  },

  _shownStack: new Array(),

  _setupHandlers: function(id, options) {
    var el = $(id);

    if (options.hideOnMouseOut) {
      BS.Event.observe(el, "mouseout", function() {
        // we should not hide popup on mouse out if there are visible popups shown after this popup
        var top = BS.Hider._shownStack[BS.Hider._shownStack.length - 1];

        //BS.Log.warn(top + " " + id);
        if (top != id) {
          return;
        }

        if (typeof options.hideOnMouseOut == 'function') {
          if (!options.hideOnMouseOut())
            return;
        }

        BS.Hider.startHidingDiv(id);
      });
    }

    BS.Event.observe(el, "mouseover", function() {
      BS.Hider.stopHidingDiv(id);
    });
  },

  _escapeHandler : function(event) {
    if (this._shownStack.length == 0) return;
    if (event.keyCode == Event.KEY_ESC) {
      var id = this._shownStack[this._shownStack.length - 1];
      BS.Hider.hideDiv(id);
      Event.stop(event);
    }
  }
};

Behaviour.addLoadEvent(function() {
  var element = document.body;
  BS.Event.observe(element, "click", function(e) {

    // Don't hide popups if click occurred inside one of the popups
    var el = Event.element(e);
    var stopOn = null;
    for(var div in BS.Hider.allDivs) {
      var elem = $(div);
      if (!elem) continue;
      if (BS.Util.descendantOf(el, elem)) {
        stopOn = elem;
        break;
      };
    }

    BS.Hider.hideAll(stopOn);
  });

  BS.Event.observe(document, "keyup", BS.Hider._escapeHandler.bindAsEventListener(BS.Hider));
});


//==========================================================================

BS.Navigation = {
    items : [],

    writeBreadcrumbs : function() {
      if (this.items.length == 0) return;

      document.write("<ul id='main_navigation' style='display: block;'>")
      document.write(this.getItemsHtml());
      document.write("</ul>");

      this._selectHeaderTabIfNeeded();
    },

    getItemsHtml: function() {
      var result = "";
      for (var i = 0; i < this.items.length - 1; i ++) {
        result += this._writeItem(this.items[i], "");
      }
      if (this.items.length > 0) {
        result += this._writeItem(this.items[this.items.length - 1], " last");
      }
      return result;
    },

    _writeItem: function(navItem, li_class) {
      if (navItem.selected) {
        li_class = "selected" + li_class;
      }
      if (navItem.itemClass) {
        li_class = li_class + " " + navItem.itemClass;
      }
      var content = navItem.title;
      if (navItem.url) {
        var idx = navItem.title.indexOf("<small>");
        if (idx >= 0) {
          content = "<a href='" + navItem.url + "'>" + navItem.title.substr(0, idx) + "</a>" + navItem.title.substr(idx);
        }
        else {
          content = "<a href='" + navItem.url + "'>" + navItem.title + "</a>";
        }
      }
      return "<li class='" + li_class +"'>" + content + "</li>";
    },

    _selectHeaderTabIfNeeded: function() {
      var navItem = this.items[0];
      if (navItem.url && navItem.url.endsWith("/admin/admin.html")) {
        this.selectAdminTab();
      }
      if (navItem.itemClass && navItem.itemClass == "project") {
        this.selectProjectsTab();
      }
    },

    selectProjectsTab: function() {
      jQuery("#overview_Tab").addClass("selected");
    },

    selectAdminTab: function() {
      jQuery("#userPanel .info:nth-child(2)").addClass("selected");
    },

    selectMySettingsTab: function() {
      jQuery("#userPanel .info:nth-child(1)").addClass("selected");
    }
};

//==========================================================================

BS.Highlight = function(element, options) {
  new Effect.Highlight(element, Object.extend(options || {}, {
    startcolor: '#ffffcc', duration: 1.0
  }));
}


BS.Logout = function(logoutUrl) {
  BS.ajaxRequest(logoutUrl, {
    onComplete: function(transport) {
      BS.XMLResponse.processRedirect(transport.responseXML);
    }
  });
}

BS.XMLResponse = {
  processModified : function(form, responseXML) {
    var rootElement = responseXML.documentElement;
    form.setModified(rootElement.firstChild && rootElement.firstChild.nodeValue == "modified");
  },

  processRedirect : function(responseXML) {
    var rootElement = responseXML.documentElement;
    var redirect = rootElement.getElementsByTagName("redirect")[0];
    if (redirect) {
      document.location.href = redirect.firstChild.nodeValue;
      return true;
    }
    return false;
  },

  processErrors : function(responseXML, errorHandlers, generalErrorHandler) {
    var eNodes = this._getErrorNodes(responseXML);
    if (!eNodes || eNodes.length == 0) return false;

    var handled = false;
    for (var i=0; i<eNodes.length; i++) {
      var elem = eNodes.item(i);
      var id = elem.getAttribute("id");
      var funcName = "on" + id.charAt(0).toUpperCase() + id.substring(1) + "Error";
      var handler = errorHandlers[funcName] || errorHandlers[id];

      if (handler && typeof(handler) == 'function') {
        handler.apply(errorHandlers, [elem]);
        handled = true;
      } else if (generalErrorHandler) {
        generalErrorHandler(id, elem);
        handled = true;
      }
    }

    return handled;
  },

  _getErrorNodes : function(responseXML) {
    var parentElement = responseXML.documentElement;
    if (parentElement == null) return null;

    var errorsNodes = parentElement.getElementsByTagName("errors");
    if (!errorsNodes || errorsNodes.length == 0) return null;
    var errorsNode = errorsNodes.item(0);
    return errorsNode.getElementsByTagName("error");
  }
}

BS.StopBuild = function(actionUrl, id, form) {
  if (form) {
    Form.disable(form);
  }

  BS.ajaxRequest(actionUrl + "?kill=" + id, {
    onComplete: function(transport) {
      setTimeout("document.location.reload()", 3000);
    }
  });

  return false;
}

BS.InstallAgentsPopup = {
  showNearElement : function(element) {
    $('installAgents').style.position = 'absolute';
    BS.Util.placeNearElement('installAgents', element, {x: -180, y: 21});
    BS.Hider.showDivWithTimeout('installAgents', {hideOnMouseOut: false});
  }
}

BS.WindowFocusMonitor = {
  startMonitor : function() {
    this.hasFocus = true;
    var that = this;

    BS.Event.observe(window, "focus", function() {
      that.onfocus();
    }.bind(this));

    BS.Event.observe(window, "blur", function() {
      that.onblur();
    }.bind(this));
  },

  onfocus : function() {
    this.hasFocus = true;
  },

  onblur : function() {
    this.hasFocus = false;
  }
};

BS.TableHighlighting = {};
BS.TableHighlighting.createInitElementFunction = function(mouseovertitle) {
  return function(element) {
    BS.Event.observe(element, "mouseover", function() {
      if (typeof(BS) == "undefined") return;
      var noTitle = element.className.indexOf("noTitle") != -1 || element.parentNode.className.indexOf("noTitle") != -1;
      var hasTitle = element.title || element.parentNode.title;
      if (!noTitle && !hasTitle) {
        element.title = mouseovertitle;
      }

      BS.Util.changeChildrenColor(this.parentNode, {
        color: '#254193',
        backgroundColor: '#ffffcc',
        filter: function(elem) {
          return elem.nodeType == 1 && elem.className.indexOf("highlight") >= 0;
        }
      });
    }.bind(element));

    BS.Event.observe(element, "mouseout", function() {
      if (typeof(BS) == "undefined") return;
      BS.Util.changeChildrenColor(this.parentNode, {
        color: '',
        backgroundColor: '',
        filter: function(elem) {
          return elem.nodeType == 1 && elem.className.indexOf("highlight") >= 0;
        }
      });
    }.bind(element));
  }
};

BS.Refreshable = {
  createRefreshFunction :function(containerId, pageUrl, passJsp) {
    return function(progressId, moreParameters, afterComplete) {
      if (BS.ServerLink && !BS.ServerLink.isConnectionAvailable()){
        BS.Log.info("Connection to server is not available. Refresh is not called.");
        return;
      }

      if (!moreParameters) {
        moreParameters = "";
      }

      var container = $(containerId);

      if (container.refresh_in_progress) {
        BS.Log.warn("Refresh already in progress for " + containerId);
        return;
      }
      container.refresh_in_progress = true;

      if (progressId && $(progressId)) {
        BS.Util.show(progressId);
      }

      var url = container.refreshUrl;
      if (!url) {
        url = pageUrl;
      }

      var myParams = passJsp ? "jsp=" + passJsp : "__fragmentId=" + containerId + "Inner";
      BS.ajaxUpdater(container, url, {
        method: 'get',
        parameters: myParams + (moreParameters ? "&" + moreParameters : ""),
        evalScripts: true,
        onFailure: function() {
          BS.Log.warn("Failure while refreshing " + containerId);
        },
        onComplete: function(transport) {
          try {
            if (progressId && $(progressId)) {
              BS.Util.hide(progressId);
            }
            if (afterComplete) {
              afterComplete();
            }
          }
          finally {
            container.refresh_in_progress = false;
          }
        }
      });
    }
  }
};


BS.WaitFor = function(condition, runWhenDone, secs2Wait) {
  if (!secs2Wait) secs2Wait = 10;
  var maxCount = secs2Wait * 1000 / 50;
  var counter = 0;

  var _waitForHandler = function() {
    if (!condition() && (maxCount > counter ++) ) {
      setTimeout(_waitForHandler, 50);
    }
    else {
      runWhenDone();
    }
  };
  _waitForHandler();
}

BS.Cleaner = {
  _toClean: {},

  add: function(id) {
    this._toClean[id] = true;
  },

  cleanAll: function() {
    for(var id in this._toClean) {
      var el = $(id);
      if (el) {
        el.onmouseover = null;
        el.onmouseout = null;
      }
    }
    this._toClean = {};
  }
}

Event.observe(window, "unload", function() {
  BS.Event.stopAll();
  BS.Cleaner.cleanAll();
  BS.PeriodicalRefresh.stop();
});

BS.PeriodicalRefresh = Class.create();
BS.PeriodicalRefresh._removeIntervalUpdate = function() {
  if (this._intervalId) {
    clearInterval(this._intervalId);
  }
}

BS.PeriodicalRefresh.start = function(interval, refreshFunc) {
  this._removeIntervalUpdate();
  this._intervalId = null;

  if (interval > 0) {
    this._intervalId = setInterval(function() { BS.reload(false, refreshFunc); }, interval * 1000);
  }
}

BS.PeriodicalRefresh.stop = function() {
  this._removeIntervalUpdate();
}

BS.InPlaceFilter = {
  apply : function(containerId, filterField, afterFilterFunc) {
    if (this.tid) {
      window.clearTimeout(this.tid);
    }

    this.tid = window.setTimeout(function() {
      var keyword = filterField.value.toUpperCase();
      if (keyword == this.prevKeyword) return;

      var narrowSearch = this.prevKeyword != null && keyword.indexOf(this.prevKeyword) != -1;
      this.prevKeyword = keyword;

      var elementsToFilter = $A($(containerId).getElementsByClassName("inplaceFiltered")); // have to copy to array because it seems the returned collection can be "live"
      for (var i=0; i<elementsToFilter.length; i++) {
        var elem = elementsToFilter[i];
        if (narrowSearch && !this.isVisible(elem)) continue;
        var text = this.getContent(elem);

        if (text.indexOf(keyword) == -1) {
          if (elem.tagName != 'OPTION' || !elem.selected) {
            if (elem.tagName == 'OPTION') {
              this.hideOption(elem, i);
            } else {
              BS.Util.hide(elem);
            }
          }
        } else {
          if (this.isHiddenOption(elem)) {
            this.showOption(elem, i);
          } else {
            BS.Util.show(elem);
          }
        }
      }

      if (afterFilterFunc) {
        afterFilterFunc();
      }

    }.bind(this), 100);
  },

  isVisible : function(elem) {
    return elem.style.display != 'none';
  },

  getContent : function(elem) {
    var content = elem._cachedContent;
    if (content != null) return content;

    content = "";
    for (var i=0; i<elem.childNodes.length; i++) {
      var node = elem.childNodes[i];
      if (node.nodeType == 1) {
        content += this.getContent(node);
      }
      if (node.nodeType == 3) {
        content += node.nodeValue;
      }
    }

    elem._cachedContent = content.replace(/[\s]+/g, ' ').strip().toUpperCase();
    return elem._cachedContent;
  },

  isHiddenOption : function(elem) {
    return elem.className && elem.className.indexOf('_hiddenOption_') != -1;
  },

  hideOption : function(elem, idx) {
    var selectElem = elem.parentNode;
    if (!selectElem._hiddenOptions) {
      selectElem._hiddenOptions = {};
    }

    selectElem._hiddenOptions[idx] = elem;

    var newElem = document.createElement("SPAN");
    selectElem.insertBefore(newElem, elem);
    selectElem.removeChild(elem);
    newElem.className = 'inplaceFiltered _hiddenOption_';
    newElem._cachedContent = this.getContent(elem);
  },

  showOption : function(elem, idx) {
    var selectElem = elem.parentNode;
    if (selectElem._hiddenOptions[idx]) {
      selectElem.insertBefore(selectElem._hiddenOptions[idx], elem);
      selectElem.removeChild(elem);

      selectElem._hiddenOptions[idx] = null;
    }
  }
};

BS.VisibilityHandlers = {
  _emptyHandler: { updateVisibility: function(){} },
  _handlers: {},

  attachTo: function(control, handler) {
    var controlId = $(control).id;
    var controlHandlers = this._handlers[controlId];
    if (controlHandlers == null) {
      controlHandlers = new Array();
      this._handlers[controlId] = controlHandlers;
    }
    controlHandlers.push(handler);
  },

  detachFrom: function(elementId) {
    this._handlers[elementId] = null;
  },

  _visibilityHandler: function(element) {
    if (!element || !element.id) return this._emptyHandler;

    var controlHandlers = this._handlers[element.id];
    if (controlHandlers == null) return this._emptyHandler;

    return {
      updateVisibility: function() {
        for (var i=0; i<controlHandlers.length; i++) {
          controlHandlers[i].updateVisibility();
        }
      }
    };
  },

  _collectElements: function(parentEl) {
    var elems = new Array();
    if (parentEl == null) return elems;

    if (parentEl.id) {
      elems.push(parentEl);
    }

    if (parentEl.id == 'mainContent') {
      for (var id in this._handlers) {
        if (!this._handlers.hasOwnProperty(id)) continue;
        elems.push($(id));
      }
    } else {
      for (var id in this._handlers) {
        if (!this._handlers.hasOwnProperty(id)) continue;
        var elem = $(id);
        if (BS.Util.descendantOf(elem, parentEl)) {
          elems.push(elem);
        }
      }
    }
    return elems;
  },

  updateVisibility: function(el) {
    if (el == null) return;
    var elems = this._collectElements($(el));
    for (var i=0; i<elems.length; i++) {
      this._visibilityHandler(elems[i]).updateVisibility();
    }
  }
};

BS.User = {
  setProperty: function(key, value, options) {
    this._setProperty(key, value, options, "setUserProperty");
  },

  deleteProperty: function(key, options) {
    this._beforeChange(options);

    BS.ajaxRequest(base_uri + '/ajax.html', {
      parameters: 'deleteUserProperty=' + key,
      onComplete: function() {
        BS.User._afterChange(options);
      }
    })
  },

  setBooleanProperty: function(key, value, options) {
    if (value) {
      this.setProperty(key, value, options);
    } else {
      this.deleteProperty(key, options);
    }
  },

  /**
   * @parem key string key property, you can get property value on the server-side using session[key] in JSP
   * @param options have keys 'progress' - ID of progress element; afterComplete - function to call after property is set
   * */
  setSessionProperty: function(key, value, options) {
    this._setProperty(key, value, options, "setSessionProperty");
  },

  _setProperty: function(key, value, options, methodName) {
    if (!options) options = {};
    this._beforeChange(options);

    BS.ajaxRequest(base_uri + '/ajax.html', {
      parameters: methodName + '=' + key + '&value=' + value,
      onComplete: function() {
        BS.User._afterChange(options);
      }
    })
  },


  _beforeChange: function(options) {
    if (options && options.progress) {
      BS.Util.show(options.progress);
    }
  },

  _afterChange: function(options) {
    options = options || {};
    if (options.progress) {
      BS.Util.hide(options.progress);
    }
    if (options.afterComplete) {
      options.afterComplete();
    }
  }
};

/**-------------------------------------------------------*/
/*-------- Simple delayed action support -----------------*/
/**-------------------------------------------------------*/
BS.DelayedAction = function(action_start, action_stop, delay) {
  if (!delay) delay = 300;
  this.delay = delay;
  this.action = action_start;
  this.stop_action = action_stop;
};

BS.DelayedAction.prototype.start = function() {
  this.timeout = setTimeout(this.action, this.delay);
};

BS.DelayedAction.prototype.stop = function() {
  clearTimeout(this.timeout);
  this.stop_action.call(this);
};

/**
 Usage:
 var progress = new BS.DelayedShow(element_id); // == new BS.DelayedShow(element_id, 300);
 progress.show();

 // some ajax call
 // onComplete: function() { progress.hide();}

 */

BS.DelayedShow = function(element) {
  BS.DelayedAction.call(this, function() {
    if ($(element)) $(element).show();
  }, function() {
    if ($(element)) $(element).hide();
  });
};

BS.DelayedShow.prototype = new BS.DelayedAction();
BS.DelayedShow.prototype.show = BS.DelayedShow.prototype.start;
BS.DelayedShow.prototype.hide = BS.DelayedShow.prototype.stop;

/**-------------------------------------------------------*/


/* Catch Javascript problems in AJAX handlers: */
Ajax.Responders.register({
  onException: function(r, e) {
    if (window.console && window.console.error) {
      console.error(e);
    }
  }
});

// Depends on jquery UFD plugin.
// See http://code.google.com/p/ufd/
BS._jQueryDropdowns = {};

BS.jQueryDropdown = function(selector) {
  var dropDown = jQuery(selector);
  var id = dropDown.attr('id');
  if (BS._jQueryDropdowns[id]) {

    // This is a tricky part.
    // BS._jQueryDropdowns[id].parentNode is a wrapper around <select> tag.
    // If this wrapper has no parent node (for instance, when node is detached and not in DOM),
    // we may have a StackOverflow problem when destroying such detached node
    // As a workaround, we add a fake parent node for this wrapper.
    if (BS._jQueryDropdowns[id].parentNode) {
      if (BS.Util.isDetached(BS._jQueryDropdowns[id].parentNode)) {
        document.createElement('div').appendChild(BS._jQueryDropdowns[id].parentNode);
      }
      jQuery(BS._jQueryDropdowns[id]).ufd("destroy");
    }
    delete BS._jQueryDropdowns[id];
  }

  BS._jQueryDropdowns[id] = $(id);

  var selectElem = dropDown[0];
  var name = selectElem.name;
  id = selectElem.id;

  dropDown.ufd({
    css: { button: BS.jQueryDropdown.namePrefix + (name ? name : id) },
    calculateZIndex: true
  });

  return dropDown;
};

// jQuery ufd creates temporary elements with names
// that ends with that prefix. We need to filter those
// fields out when sending parameters to the server.
BS.jQueryDropdown.namePrefix = "-udf-teamcity-ui";

BS.jQueryDropdown.setJQueryOptions = function($) {
  $.ui.ufd.defaults.skin = "sexy";
  $.ui.ufd.defaults.prefix = BS.jQueryDropdown.namePrefix;
  $.ui.ufd.prototype.options = $.ui.ufd.defaults; // 1.8 default options location
};

// Moved from allProjects.js to reuse in all pages.
BS.openUrl = function(event, url) {
  if (Event.element(event) && Event.element(event).tagName == 'A') {
    // For safari to process links as usual
    return true;
  } else {
    if (Event.isLeftClick(event)) {
      document.location.href = url;
    }
  }
  return false;
};
