
/*
 * Copyright 2000-2011 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

BS.AbstractModalDialog = {
  _attachedToRoot: false,

  /** should return $('element id') for modal dialog element **/
  getContainer : function() {},

  beforeShow : function() {},

  afterClose : function() {},

  getDragHandle : function() {
    var dialog = $(this.getContainer());
    var handle = dialog;
    if (dialog != null) {
      for (var i=0; i<dialog.childNodes.length; i++) {
        var childNode = dialog.childNodes[i];
        if (childNode.nodeType == 1) { // if element
          handle = childNode;
          break;
        }
      }
    }

    return handle;
  },

  bindCtrlEnterHandler : function(handler) {
    this._ctrlEnterHandler = function(event) {
      // keyCode is 10 at least in Chrome and IE, 13 in FF.
      if (event.ctrlKey && (/*isIE &&*/ event.keyCode == 10 || event.keyCode == Event.KEY_RETURN)) {
        handler();
      }
    }.bindAsEventListener(this);

    Event.observe(document, "keypress", this._ctrlEnterHandler, false);
  },

  unbindCtrlEnterHandler : function() {
    if (!this._ctrlEnterHandler) return;
    Event.stopObserving(document, "keypress", this._ctrlEnterHandler, false);
    this._ctrlEnterHandler = null;
  },

  _removeDuplicateElements: function() {
    var el = $(this.getContainer());
    var origId = el.id;
    el.id = '_' + origId;
    var dup = $(origId);
    while(dup) {
      dup.parentNode.removeChild(dup);
      dup = $(origId);
      //alert('rem');
    };

    el.id = origId;
  },

  _fixElementPlacement: function() {
    this._removeDuplicateElements();
    if (this._attachedToRoot) return;

    var dialog = $(this.getContainer());

    var parent = dialog.parentNode;
    var needToReattach = false;
    while (parent != document.documentElement && parent.id != 'mainContent') {
      if (parent.style && (parent.style.position || parent.style.display == 'none')) {
        needToReattach = true;
        break;
      }
      parent = parent.parentNode;
    }

    if (needToReattach) {
      // we have to append dialog to the main container, otherwise position might not be calculated correctly
      var toMove = dialog;
      parent = dialog.parentNode;
      if (parent.tagName == 'FORM' && parent.id && dialog.id == parent.id + "Dialog") {
        toMove = parent;
        parent = parent.parentNode;
      }

      if (toMove.parentNode != $('mainContent')) {
        toMove.parentNode.removeChild(toMove);
        $('mainContent').appendChild(toMove);
      }

      this._attachedToRoot = true;
    }
  },

  showAt : function(x, y) {
    this._fixElementPlacement();

    var dialog = $(this.getContainer());
    this.beforeShow();
    BS.Util.place(dialog, x, y);

    var afterHide = function() {
      var div = $(this._overlappingDivId());
      if (div) {
        $('mainContent').removeChild(div);
      }
      this.afterClose();
      this.unbindCtrlEnterHandler();
    }.bind(this);

    BS.Hider.showDivWithTimeout(dialog.id, {
      hideOnMouseOut: false,
      hideOnMouseClickOutside: false,
      afterHideFunc: afterHide,
      draggable: true,
      dragHandle: this.getDragHandle()
    });

    var overlappingDiv = this.getOverlappingDiv();
    var dialogZindex = dialog.style.zIndex;
    overlappingDiv.style.zIndex = "" + (parseInt(dialogZindex, 10) - 1);
    if (!BS.Util.visible(overlappingDiv)) {
      BS.Util.show(overlappingDiv);
    }
  },

  isVisible : function() {
    return BS.Util.visible(this.getContainer());
  },

  showCentered : function(container) {
    this._fixElementPlacement();

    var dialog = $(this.getContainer());
    var pos = BS.Util.computeCenter(dialog, container);
    this.showAt(pos[0], pos[1]);
  },

  recenterDialog: function(container) {
    this._fixElementPlacement();

    var dialog = $(this.getContainer());
    var dialogPos = Position.positionedOffset(dialog);
    var pos = BS.Util.computeCenter(dialog, container);
    if (dialogPos[1] > pos[1]) {
      BS.Util.place(dialog, pos[0], pos[1]);
      this.updateDialog();
    }
  },

  close : function() {
    this.doClose();
  },

  doClose: function() {
    if (this.getContainer()) {
      BS.Hider.hideDiv(this.getContainer().id);
    }
  },

  _overlappingDivId: function() {
    return 'overlappingDivId';
  },

  getOverlappingDiv : function() {
    var id = this._overlappingDivId();
    var overlappingDiv = $(id);
    if (overlappingDiv) {
      return overlappingDiv;
    }

    overlappingDiv = document.createElement("div");
    overlappingDiv.setAttribute("id", id);
    overlappingDiv.style.display = "none";
    overlappingDiv.style.position = "absolute";
    if (navigator.userAgent.indexOf("Opera") ==-1 ){
      overlappingDiv.style.backgroundColor = '#fff';
    }

    Element.setOpacity(overlappingDiv, 0.3);
    $('mainContent').appendChild(overlappingDiv);

    Position.absolutize(overlappingDiv);
    Position.clone($('mainContent'), overlappingDiv);

    var footerHeight = Element.getHeight($('footer'));

    overlappingDiv.style.height = (Element.getHeight(overlappingDiv) + footerHeight) + 'px';

    return overlappingDiv;
  },

  // updates dialog iframe if needed
  updateDialog: function() {
    BS.Util.moveDialogIFrame(this.getContainer());
  }
};

BS.DialogWithProgress = OO.extend(BS.AbstractModalDialog, {
  showProgress: function(event) {
    BS.ProgressPopup.showProgress(Event.element(event), "Loading...", {shift: {x: 15, y: 20}});
  },

  hideProgress: function() {
    BS.ProgressPopup.hidePopup(0, true);
  },

  showCentered: function() {
    this.hideProgress();
    BS.AbstractModalDialog.showCentered.apply(this, arguments);
  },

  showAt: function() {
    this.hideProgress();
    BS.AbstractModalDialog.showAt.apply(this, arguments);
  }
});
