
/*
 * Copyright 2000-2011 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var BS;

BS.PluginPropertiesForm = OO.extend(BS.AbstractWebForm, {

  serializeParameters : function() {
    var params = BS.Util.serializeForm(this.formElement());

    var passwordFields = Form.getInputs(this.formElement(), "password");
    if (!passwordFields) return params;

    for (var i=0; i<passwordFields.length; i++) {
      var name = passwordFields[i].name;
      if (name.indexOf("prop:") != 0) continue;
      var encryptedName = "prop:encrypted:" + passwordFields[i].id;

      params += "&" + encryptedName + "=";
      var encryptedValue = $(encryptedName).value;
      if (encryptedValue == "" || passwordFields[i].value != passwordFields[i].defaultValue) {
        encryptedValue = BS.Encrypt.encryptData(passwordFields[i].value, this.formElement().publicKey.value);
      }

      params += encryptedValue;
    }

    return params;
  },

  propertiesErrorsHandler : function(id, elem) {
    BS.PluginPropertiesForm.showError(id, elem.firstChild.nodeValue);
  },

  showError : function(id, message) {
    if ($("error_" + id)) {
      $("error_" + id).innerHTML = message.escapeHTML();
    }
    if ($(id) && $(id).type && ($(id).type == 'text' || $(id).type == 'password')) {
      $(id).className = $(id).className + " errorField";
    }
  }
})

// contains names of all multiline properties on the page
BS.MultilineProperties = {

  _init : function() {
    if (!this._properties) {
      this._properties = new Array();
    }
  },

  _findProperty : function(name) {
    for (var i=0; i<this._properties.length; i++) {
      var prop = this._properties[i];
      if (prop.name == name) {
        return prop;
      }
    }

    return null;
  },

  clearProperties : function() {
    this._properties = null;
  },

  addProperty : function(name, initFunc) {
    this._init();

    var found = this._findProperty(name);

    if (found) {
      found.visible = false;
      found.initFunc = initFunc;
    } else {
      var prop = {};
      this._properties.push(prop);
      prop.visible = false;
      prop.name = name;
      prop.initFunc = initFunc;
    }
  },

  setVisible : function(name, visible) {
    var found = this._findProperty(name);
    if (found) {
      found.visible = visible;
    }
  },

  isVisible : function(name) {
    var found = this._findProperty(name);
    if (found) {
      return found.visible;
    }

    return false;
  },

  updateVisible : function() {
    try {
      if (!this._properties) return;

      for (var i = 0; i < this._properties.length; i++) {
        var prop = this._properties[i];
        if (!prop.visible) continue;
        this.doShow(prop.name, false);
      }
    } finally {
      BS.VisibilityHandlers.updateVisibility($('mainContent'));
    }
  },

  show : function(name, focus) {
    this.doShow(name, focus);
    BS.MultilineProperties.setVisible(name, true);
    BS.MultilineProperties.updateVisible();
  },

  doShow: function(name, focus) {
    var found = this._findProperty(name);
    if (!found) return;

    var container = $(name + '_Container');
    if (!container) return; // todo: remove property from collection

    if (found.initFunc) {
      found.initFunc();
    }

    var linkContainer = $(name + '_LinkContainer');
    if (linkContainer) {
      BS.Util.hide(linkContainer);
    }
    BS.Util.show(name + '_NoteContainer');

    var textarea = $(name);
    var corner = $(name + '_DragCorner');
    var hideLink = $(name + '_HideLink');

    if (!container) return;

    BS.Util.show(container);
    BS.Util.show(hideLink);
    BS.Util.show(corner);

    container.style.width = (textarea.offsetWidth + 1) + 'px';
    container.style.height = (textarea.offsetHeight + corner.height + 3) + 'px';

    container.recalculateOffset();
    corner.draw(container);
    hideLink.draw(container);

    if (focus && !textarea.disabled) {
      textarea.focus();
    }
  },

  hide : function(name) {
    BS.Util.hide(name + '_Container');
    BS.Util.hide(name + '_HideLink');
    BS.Util.hide(name + '_DragCorner');
    BS.Util.hide(name + '_NoteContainer');
    BS.Util.show(name + '_LinkContainer');
    this.updateImage(name);

    BS.MultilineProperties.setVisible(name, false);
    BS.MultilineProperties.updateVisible();
  },

  updateImage : function(name) {
    var image = $('image_' + name);
    if ($(name).value != '') {
      image.src = image._notEmptyPath;
    } else {
      image.src = image._emptyPath;
    }
  }
};

Event.observe(window, "resize", function() {
  BS.MultilineProperties.updateVisible();
});