/*
 * Copyright 2000-2011 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * This file adds CSS classes to BODY element of the document, corresponding to the current view area width.
 * The classes can be tuned by CSS_WIDTHS constant.
 *
 *
 * @author kir
 */

(function() {

  // If browser width is 810 px, this code will set classes gt480 gt800 lt1200
  // If browser width is 2000 px, this code will set classes gt480 gt800 gt1200
  // If browser width is 200 px, this code will set classes lt480 lt800 lt1200
  var CSS_WIDTHS = [800, 1000, 1200];


  var timeout;
  var update_classes = function() {
    timeout = null;

    var body = document.body;
    var classNames = $A(body.classNames());
    var currWidth = document.viewport.getWidth();
    for (var i = 0; i < CSS_WIDTHS.length; i++) {
      var w = CSS_WIDTHS[i];
      classNames = classNames.without('lt' + w);
      classNames = classNames.without('gt' + w);
      if ( currWidth < w ) {
        classNames.push('lt' + w);
      }
      else {
        classNames.push('gt' + w);
      }
    }
    body.className = classNames.join(' ');
  };

  var onresize = function() {
    if (timeout) {
      return;
    }
    timeout = setTimeout(update_classes, 500);
  };

  Event.observe(window, "load", onresize);
  Event.observe(window, "resize", onresize);
  Event.observe(window, "unload", function() {
    Event.stopObserving(window, "resize", onresize);
  });
})();

